#!/bin/bash
echo "YamahaSteinbergUSBControlPanel.sh start."
CONTROL_PANEL_NAME="Yamaha Steinberg USB Control Panel"
CONTROL_PANEL_BUNDLE="/Applications/${CONTROL_PANEL_NAME}.app"
CONTROL_PANEL_PLIST="${CONTROL_PANEL_BUNDLE}/Contents/Info.plist"
CONTROL_PANEL_BINARY="${CONTROL_PANEL_BUNDLE}/Contents/MacOS/${CONTROL_PANEL_NAME}"
UNINSTALLER=$(echo "$0" | awk '{sub(".app.*", ".app");print $0;}')

# from "UninstallController.h"
ExecEnumTypeDEF=0
ExecEnumTypeNOCP=1
ExecEnumTypeDefError=2
ExecEnumTypeNoDext=3
ExecEnumTypeDAError=4
ExecEnumTypeDeactivated=5
ExecEnumTypeCPRemains=6

# from "SystemExtensions.h" OSSystemExtensionErrorCode
OSSystemExtensionErrorUnknown=1
OSSystemExtensionErrorMissingEntitlement=2
OSSystemExtensionErrorUnsupportedParentBundleLocation=3
OSSystemExtensionErrorExtensionNotFound=4
OSSystemExtensionErrorExtensionMissingIdentifier=5
OSSystemExtensionErrorDuplicateExtensionIdentifer=6
OSSystemExtensionErrorUnknownExtensionCategory=7
OSSystemExtensionErrorCodeSignatureInvalid=8
OSSystemExtensionErrorValidationFailed=9
OSSystemExtensionErrorForbiddenBySystemPolicy=10
OSSystemExtensionErrorRequestCanceled=11
OSSystemExtensionErrorRequestSuperseded=12
OSSystemExtensionErrorAuthorizationRequired=13

EXIT_VAL=$ExecEnumTypeDEF

if [ -d "${CONTROL_PANEL_BUNDLE}" ]; then
    CONTAINSDRIVEREXTENSION=$(defaults read "${CONTROL_PANEL_PLIST}" ContainsDriverExtension 2>/dev/null)
    if [ $? -eq 0 ]; then
        # Driver Extension が Control Panel に含まれている場合 = plist に key ContainsDriverExtension が存在している場合 = 終了ステータスが 0 以外の場合
        if [ "${CONTAINSDRIVEREXTENSION}" -ne 0 ]; then
#             # key ContainsDriverExtension が YES の場合
            echo "contains driver extension."
            echo "${CONTROL_PANEL_BINARY} -deactivate"
            "${CONTROL_PANEL_BINARY}" -deactivate
            ret=$?
            case ${ret} in
            "0")
                # コンパネでdeacitivate完了
                # deactivateされているだけなので、コンパネは残る
				EXIT_VAL=$ExecEnumTypeDeactivated
				echo "EXIT_VAL=ExecEnumTypeDeactivated"
				;;
			"$OSSystemExtensionErrorExtensionNotFound")
                # コンパネからエラー（deactivate済）
                EXIT_VAL=$ExecEnumTypeDeactivated
	            echo "EXIT_VAL=ExecEnumTypeDeactivated"
	            ;;
			"$OSSystemExtensionErrorAuthorizationRequired")
                # コンパネからエラー（deactivate権限昇格キャンセル）
#                 EXIT_VAL=$ExecEnumTypeCPRemains
# 	            echo "EXIT_VAL=ExecEnumTypeCPRemains"
                # キャンセルした場合もメッセージを表示するよう変更
                EXIT_VAL=$ExecEnumTypeDAError
	            echo "EXIT_VAL=ExecEnumTypeDAError"
	            ;;
			*)
                # コンパネからエラー コンパネは残る
                # deactivate 済、もしくはdeactivate 出来ない異常な状態
                # OSSystemExtensionErrorExtensionNotFound(4)以外のエラーは全てメッセージ表示
                EXIT_VAL=$ExecEnumTypeDAError
	            echo "EXIT_VAL=ExecEnumTypeDAError"
	            ;;
			esac

        else
            # CONTAINSDRIVEREXTENSION の値が０
            EXIT_VAL=$ExecEnumTypeNoDext
            echo "EXIT_VAL=ExecEnumTypeNoDext"
            echo "no driver extension."
        fi
    else
        # CONTAINSDRIVEREXTENSION Keyがない（旧コンパネ）
        EXIT_VAL=$ExecEnumTypeDefError
        echo "EXIT_VAL=ExecEnumTypeDefError"
        echo "no entry."
    fi
else
    # コンパネ がない（アンインストーラー再施行等）
    EXIT_VAL=$ExecEnumTypeNOCP
    echo "EXIT_VAL=ExecEnumTypeNOCP"
    echo "${CONTROL_PANEL_BUNDLE} does not exist."
fi

open -a "${UNINSTALLER}"

echo "YamahaSteinbergUSBControlPanel.sh end."

exit $EXIT_VAL
